package view
{
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;
	import flash.text.*;
	import flash.utils.*;
	import jp.nium.core.debug.Logger;
	import jp.progression.casts.*;
	import jp.progression.commands.*;
	import jp.progression.commands.display.AddChild;
	import jp.progression.scenes.*;
	import sound.SoundManager;
	import ui.DefalutBtn;
	
	/**
	 * Footer
	 * @author Yasu (clockmaker.jp)
	 */
	public class Footer extends CastSprite
	{
		[Embed(source = '/assets/foot_btn_full.png')] private var FullImg:Class;
		[Embed(source = '/assets/foot_btn_sound_on.png')] private var SoundOnImg:Class;
		[Embed(source = '/assets/foot_btn_sound_off.png')] private var SoundOffImg:Class;
		[Embed(source = '/assets/foot_copyright.png')] private var Copyright:Class;
		
		private var bg:Sprite;
		private var fullBtn:CastButton;
		private var soundBtn:CastButton;
		private var copyright:Bitmap;
		private var footText:Bitmap;
		
		private var soundOn:Bitmap = new SoundOnImg();
		private var soundOff:Bitmap = new SoundOffImg();
		private var tf:TextField;
		private var timer:Timer;
		
		public function Footer() 
		{
			bg = new Sprite();
			
			fullBtn = new DefalutBtn() as CastButton;
			fullBtn.addChild(new FullImg);
			fullBtn.addEventListener(MouseEvent.CLICK, fullScreenHandler)
			fullBtn.toolTip.text = "Full Screen"
			
			soundBtn = new DefalutBtn() as CastButton;
			soundBtn.addChild(soundOn);
			soundBtn.addChild(soundOff);
			soundBtn.addEventListener(MouseEvent.CLICK, soundBtnHandler)
			soundBtn.toolTip.text = "Sound"
			soundOn.visible = true;
			soundOff.visible = false;
			
			copyright = new Copyright;
			
			var tformat:TextFormat = new TextFormat("Arial", 9);
			
			tf = new TextField();
			tf.textColor = 0xFFFFFF;
			tf.defaultTextFormat = tformat;
			tf.antiAliasType = AntiAliasType.ADVANCED;
			tf.selectable = false;
			
		}
		
		private function soundBtnHandler(e:MouseEvent):void 
		{
			SoundManager.instance.isMute = !SoundManager.instance.isMute;
			
			soundOn.visible = !SoundManager.instance.isMute;
			soundOff.visible = SoundManager.instance.isMute;
		}
		
		private function fullScreenHandler(e:MouseEvent):void 
		{
			if (stage.displayState == StageDisplayState.NORMAL)
				stage.displayState = StageDisplayState.FULL_SCREEN
			else
				stage.displayState = StageDisplayState.NORMAL
		}
		
		override protected function atCastAdded():void 
		{
			addCommand(
				new AddChild(this, bg),
				new AddChild(this, fullBtn),
				new AddChild(this, soundBtn),
				new AddChild(this, copyright),
				new AddChild(this, tf),
				function():void
				{
					stage.addEventListener(Event.RESIZE, resizeHandler);
					resizeHandler();
				}
			)
			
			timer = new Timer(50);
			timer.addEventListener(TimerEvent.TIMER, tick);
			timer.start();
		}
		
		private function tick(e:TimerEvent):void
		{
			tf.text = "TIME: " + getDigitalTime(getTimer());
		}
		
		/**
		 * ミリ秒を渡すとデジタル時計を文字型で返す
		 * @param	time
		 * @return
		 */
		private function getDigitalTime(time:int):String
		{
			var milStr:String;
			var minStr:String;
			var secStr:String;
			
			var mil:Number = Math.floor(time);
			var sec:Number = Math.floor(mil / 1000);
			var min:Number = Math.floor(sec / 60);
			
			if(min > 99)	min = 99;
			sec %= 60;
			mil %= 1000;
			
			minStr = (min < 10) ? "0" + min : String(min);
			secStr = (sec < 10) ? "0" + sec : String(sec);
			
			var z:String = "";
			if(mil < 100) z += "0";
			if(mil < 10)  z += "0";
			milStr = z + mil;
			
			return minStr + ":" + secStr + ":" + milStr;
		}
		
		private function resizeHandler(e:Event = null):void 
		{
			bg.y = stage.stageHeight - 25;
			bg.graphics.clear();
			bg.graphics.beginFill(0x0, 1);
			bg.graphics.drawRect(0, 0, stage.stageWidth, 25);
			
			copyright.x = stage.stageWidth - copyright.width - 20;
			copyright.y = stage.stageHeight - 21;
			
			fullBtn.x = 20;
			fullBtn.y = stage.stageHeight - 22;
			
			soundBtn.x = 120;
			soundBtn.y = stage.stageHeight - 22;
			
			tf.x = 220;
			tf.y = stage.stageHeight - 19;
		}
	}
}