package view {
	import assets.*;
	import caurina.transitions.*;
	import flash.display.*;
	import flash.events.*;
	import flash.filters.*;
	import flash.ui.Keyboard;
	import jp.nium.core.debug.Logger;
	import jp.progression.commands.*;
	import jp.progression.commands.lists.*;
	import jp.progression.commands.managers.*;
	import jp.progression.commands.tweens.*;
	import jp.progression.scenes.*;
	import org.papervision3d.cameras.*;
	import org.papervision3d.core.math.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.materials.utils.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.objects.primitives.*;
	import org.papervision3d.view.*;

	public class World extends BasicView {
		public static const FRAMERATE_MAX:int = 60;
		public static const FRAMERATE_MIN:int = 12;
		public static const SLIDE_WIDTH:uint = 1024;
		public static const SLIDE_HEIGHT:uint = 768;
		public static const MARGIN_WIDTH:uint = 600;
		public static const MARGIN_HEIGHT:uint = 600;
		private static var _instance:World = new World();

		private var _curretPage:int = 0; // 現在のページ
		private var _planes:Vector.<Plane>;
		private var _root:DisplayObject3D;
		private var _isLock:Boolean = false; // アニメーション中のロック制御のためのフラグ
		private var _wheelPage:int = 0;
		private var _slideNo:SlideNo = new SlideNo();

		// FLash CS5で作ったムービークリップのインスタンス(SWCファイルにて出力済み)を呼び出す
		private static const SLIDE_CLIP:MovieClip = new SlideClip();

		public static function get numSlides():uint {
			return SLIDE_CLIP.totalFrames;
		}

		/**
		 * 新しい World インスタンスを作成します。
		 */
		public function World(){
			super(SLIDE_WIDTH, SLIDE_HEIGHT, false, false, CameraType.FREE);
			
			if (stage)
				init(null);
			else
				addEventListener(Event.ADDED_TO_STAGE, init);
		}

		/**
		 * 初期化
		 * @param	e
		 */
		private function init(e:Event):void {
			removeEventListener(Event.ADDED_TO_STAGE, init);
			
			Logger.info("hogehoge");
			startRendering();
			
			viewport.filters = [new DropShadowFilter(4, 90, 0x0, 1, 16, 16, 1, 2)];

			_root = new DisplayObject3D();
			scene.addChild(_root);

			_planes = new Vector.<Plane>();
			
			for (var i:int = 0; i < SLIDE_CLIP.totalFrames; i++){
				SLIDE_CLIP.gotoAndStop(i + 1);

				var bmd:BitmapData = new BitmapData(SLIDE_WIDTH, SLIDE_HEIGHT, false);
				bmd.draw(SLIDE_CLIP);

				var m:BitmapMaterial = new BitmapMaterial(bmd);
				var p:Plane = new Plane(m, SLIDE_WIDTH, SLIDE_HEIGHT, 1, 1);
				_root.addChild(p);
				p.x = 10000 * Math.random() - 5000;
				p.y = 10000 * Math.random() - 5000;

				_planes[i] = p;
			}

			// カメラのズーム値、フォーカス値
			camera.zoom = 4;
			camera.focus = SLIDE_HEIGHT / 4;
		}

		/**
		 * 横一列に整列
		 */
		public function getMoveLineCommand():Command {
			
			var sList:SerialList = new SerialList();
			var tList:TweenList = new TweenList(1.5);
			var p:Plane,
				i:int;
			
			var targetSlide:Plane = _planes[_curretPage];
			
			tList.addCommand(
				new DoTweener(_root, { rotationY : 0, time: 3, transition: "easeInOutExpo"}),
				new DoTweener(camera, { 
						x: (SLIDE_WIDTH + MARGIN_WIDTH) * (_curretPage - World.numSlides / 2), 
						y: 0,
						//z: - SLIDE_HEIGHT * 2,
						rotationX: 0, 
						time: 4, 
						transition: "easeInOutExpo"})
				);
			
			for (i = 0; i < World.numSlides; i++){
				p = _planes[i];
				
				tList.addCommand(
					new DoTweener(p, { 
						x          : (SLIDE_WIDTH + MARGIN_WIDTH) * (i - World.numSlides / 2), 
						y          : 0, 
						z          : 0, 
						rotationY  : 0, 
						time       : 2,
						transition : "easeInOutExpo"
						} )
					);
			}
			
			sList.addCommand(
				function():void {
					removeChild(_slideNo);
					stage.removeEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
					viewport.removeEventListener(MouseEvent.CLICK, onClick);
					stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyUp);
				},
				tList,
				function():void {
					for (i = 0; i < World.numSlides; i++){
						p = _planes[i];
						BitmapMaterial(p.material).doubleSided = false;
						BitmapMaterial(p.material).precise = false;
					}
					
					stopRendering();
					stage.quality = StageQuality.HIGH;
					stage.frameRate = FRAMERATE_MIN;
					singleRender();
				}
				);
				
			return sList;
		}

		/**
		 * 円周に整列
		 */
		public function getMoveCarouselCommand():Command {
			var sList:SerialList = new SerialList();
			var tList:TweenList = new TweenList(2);
			
			var len:Number = (SLIDE_WIDTH + MARGIN_WIDTH) * World.numSlides / Math.PI / 2;

			tList.addCommand(
				new SerialList(null,
					new DoTweener(camera, {
						x: 0,
						y: 4000,
						z: -len * 1.5, 
						rotationX: 35, 
						time: 4,
						transition: "easeInOutExpo"
					}),
					new DoTweener(camera, {
						x: -len * 4 / 5, 
						y: 500,
						z: -len * 4 / 5, 
						rotationX: 10, 
						time: 4, 
						transition: "easeInOutExpo"
						})
					)
			);
			
			for (var i:int = 0; i < World.numSlides; i++){
				var p:Plane = _planes[i];

				var radiun:Number = i / World.numSlides * Math.PI * 2;
				BitmapMaterial(p.material).doubleSided = true;
				BitmapMaterial(p.material).precise = true;
				
				tList.addCommand(
					new DoTweener(p, { 
						x              : len * Math.sin(radiun),
						y              : 0, 
						z              : len * Math.cos(radiun),
						rotationY      : radiun * Number3D.toDEGREES + 180, 
						time           : 5,
						transition     : "easeInOutExpo"
						} )
					);
			}
			
			sList.addCommand(
				function():void {
					// レンダリングスイッチ
					startRendering();
					stage.quality = StageQuality.LOW;
					stage.frameRate = FRAMERATE_MAX;
				},
				tList,
				function():void {
					// スライドナンバーを表示
					addChild(_slideNo);
					updateWheel();
					
					// マウスイベント
					stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
					viewport.addEventListener(MouseEvent.CLICK, onClick);
					stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyUp);
				}
				);
				
			return sList;
		}
		
		private function onKeyUp(e:KeyboardEvent):void 
		{
			switch(e.keyCode)
			{
				case Keyboard.ENTER: onClick(null); break;
				case Keyboard.RIGHT:
				case Keyboard.UP:
					_wheelPage++; updateWheel(); break;
				case Keyboard.LEFT:
				case Keyboard.DOWN:
					_wheelPage--; updateWheel(); break;
			}
		}
		
		/**
		 * クリックされたとき
		 * @param	e
		 */
		private function onClick(e:Event):void 
		{
			new Goto(new SceneId("/index/" + _wheelPage)).execute();
		}

		/**
		 * マウスホイールが動いたとき
		 * @param	e
		 */
		private function onMouseWheel(e:MouseEvent):void {
			// ホイールページの更新
			_wheelPage += (e.delta > 0) ? 1 : -1;
			updateWheel();
		}
		
		private function updateWheel():void
		{
			_wheelPage = Math.max(0, Math.min(World.numSlides - 1, _wheelPage));
			
			// テキスト表示
			_slideNo.num_txt.text = (_wheelPage + 1) + "/" + World.numSlides;
			
			// 回転
			Tweener.addTween(_root, { rotationY: - _wheelPage / World.numSlides * 360 + 235, time: 1 } );
			_curretPage = _wheelPage;
		}

		public function moveSlide(targetPage:int, backFlag:Boolean = true ):Command {
			var currentSlide:Plane = _planes[targetPage];
			
			var sList:SerialList = new SerialList();
			sList.addCommand(
				function():void {
					// レンダリング開始
					startRendering();
					stage.quality = StageQuality.LOW;
					stage.frameRate = FRAMERATE_MAX;
				});
			
			
			Logger.info("backFlag", backFlag);
			if(backFlag){
				sList.addCommand(
					// カメラを引きの位置に移動させる
					new DoTweener(camera, {
						z: -1500, 
						time: 0.8, 
						transition: "easeInOutExpo" } ),
					// カメラをスライドの位置に移動させる
					new DoTweener(camera, {
						x: currentSlide.x, 
						y: currentSlide.y, 
						time: 1, 
						transition: "easeInOutExpo"})
						);
			}
			
			sList.addCommand(
				// カメラを初期の位置に遅れて戻す
				new DoTweener(camera, {
					z: -SLIDE_HEIGHT, 
					time: 0.9, 
					transition: "easeInOutExpo"
					}),
				function():void {
					//  レンダリング停止
					stopRendering();
					stage.quality = StageQuality.HIGH;
					stage.frameRate = FRAMERATE_MIN;

					// 停止したタイミングのレンダリング
					singleRender();
				}
				);
				
			return sList;
		}

		static public function get instance():World {
			return _instance;
		}
	}
}
